package net.sf.sql2java.common.parameters;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.sql2java.common.parameters.handlers.ParameterHandler;
import net.sf.sql2java.utils.StringUtils;

import org.apache.log4j.Logger;

public class StringParser {
	public static final Logger LOGGER = Logger.getLogger(StringParser.class);
    
    private ParameterHandler handler;
    
// -- Constructors ------------------------------------------------------------
    
    public StringParser() {
    	this(null);
    }
    
    public StringParser(ParameterHandler handler) {
    	this.handler = handler;
    }
    
// -- Parameter Parsing -------------------------------------------------------
    
    /**
     * Parse the given input <code>String</code>.
     * <p>
     * The given input String will be parsed and searched for parameters. A
     * parameter will have the following layout:
     * <p>
     * <b>:{parameter_name}</b>
     * <p>
     * If a parameter is found, the <code>ParameterHandler</code> is called.
     * This handler will act according to it's implementation.
     * 
     * @param input	The input we want to parse.
     * @return	The input String, converted and modified by the 
     * 			<code>ParameterHandler</code>
     * @see ParameterHandler
     */
    public String parse(String input) {
    	// -- return if input was null
    	if (input == null) return null;
    	
    	// -- create the result object
    	String result = input;
    	
    	// -- create the pattern to look for
    	Pattern pattern = Pattern.compile(StringUtils.parameterize(null), Pattern.CASE_INSENSITIVE);
        
    	// -- look for the pattern
        Matcher matcher = pattern.matcher(result);
        
        // -- iterate the matches
        while (matcher.find()) {
        	// -- get the next parameter
        	String parameter = matcher.group();
        	
        	parameter = parameter.substring(2, parameter.length() - 1);
        
        	// -- so we found a parameter. Let's notify the handler so he can
        	// -- take actions.
        	result = handler.onParameter(result, parameter);
        }
        
        // -- in the end, return the result
        return result;
    }
    
    public StringBuffer parse(StringBuffer input) {
    	String result = parse(input.toString());
    	
    	return new StringBuffer(result);
    }
    
// -- Getters -----------------------------------------------------------------	
    
    public ParameterHandler getHandler() {
		return handler;
	}
    
// -- Setters -----------------------------------------------------------------
    
    /**
     * Set the <code>ParameterHandler</code>
     * 
     * @param handler	A valid parameter handler. This parameter can't be 
     * 					<code>null</code>
     */
    public void setHandler(ParameterHandler handler) {
		this.handler = handler;
	}
}
