package net.sf.sql2java.common.parameters.handlers;

import java.util.Map;

import net.sf.sql2java.conversion.KeywordGroup;
import net.sf.sql2java.utils.StringUtils;
import ognl.Ognl;
import ognl.OgnlContext;
import ognl.OgnlException;

import org.apache.log4j.Logger;

public class ContextParameterHandler implements ParameterHandler {
	public static final Logger LOGGER = Logger.getLogger(ContextParameterHandler.class);
	private OgnlContext context;

// -- Constructors ------------------------------------------------------------
	
	public ContextParameterHandler(Map<String, ?> context) {
		this.context = new OgnlContext(context);
	}
	
// -- Elegible Methods --------------------------------------------------------
	
	@Override
	public boolean isElegible(String parameter) {
		return true;
	}
	
// -- Parameter Substitution --------------------------------------------------

	@Override
	public String onParameter(String content, String parameter) {
		// -- create the result
		String result = content;
		
		// -- check if a convertor is needed
		KeywordGroup translator = StringUtils.getKeywordGroup(parameter);
		if (translator != null) parameter = parameter.substring((translator.getName() + ":").length());
		
		// -- separate the navigation expression from the modifiers
		String[] parts = parameter.split("@");
		
		// -- the first part would be the navigation expression.
		String expression = parts[0];

		// -- get the root
		Object root = searchRoot(parameter);
		
		if (root == null) return result;
		
		try {
			// -- evaluate the expression
			Object value = Ognl.getValue(expression, context);
			
			// -- return if the value was null
			if (value == null) return result;
			
			// -- let's find our object
			String replacement = (translator == null) ? value.toString() : translator.translate(value.toString());
			
			// -- apply the modifiers onto the replacement
			replacement = StringUtils.performModifiers(replacement, parts);
			
			// -- replace all occurrences of the parameter with the replacement
			result = result.replaceAll(
					StringUtils.parameterize(parameter), 
					replacement
			);
			
			// -- return the result
			return result;
			
		} catch (OgnlException oe) {
			LOGGER.warn(
					String.format("Unable to lookup parameter %s: ", parameter), 
					oe
			);
			
			return content;
		}
	}
	
	private Object searchRoot(String parameter) {
		Object result = null;
		
		String key = parameter.split("\\.")[0];
		
		result = context.get(key);
		
		return result;
	}
}
